@@ -1,18 +1,21 @@
-# Copyright 2012 Google Inc.
-#
-# http://www.logilab.fr/ -- mailto:contact@logilab.fr
-# This program is free software; you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the Free Software
-# Foundation; either version 2 of the License, or (at your option) any later
-# version.
-#
-# This program is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details
-#
-# You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# -*- coding: utf-8 -*-
+# Copyright (c) 2013-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
+# Copyright (c) 2013-2014 Google, Inc.
+# Copyright (c) 2014-2017 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2014 Cosmin Poieana <cmin@ropython.org>
+# Copyright (c) 2014 Vlad Temian <vladtemian@gmail.com>
+# Copyright (c) 2014 Arun Persaud <arun@nubati.net>
+# Copyright (c) 2015 Cezar <celnazli@bitdefender.com>
+# Copyright (c) 2015 Chris Rebert <code@rebertia.com>
+# Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
+# Copyright (c) 2016 Jared Garst <cultofjared@gmail.com>
+# Copyright (c) 2017 Martin <MartinBasti@users.noreply.github.com>
+# Copyright (c) 2017 Christopher Zurcher <zurcher@users.noreply.github.com>
+# Copyright (c) 2017 Łukasz Rogalski <rogalski.91@gmail.com>
+
+# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+# For details: https://github.com/PyCQA/pylint/blob/master/COPYING
+
 """Checkers for various standard library functions."""
 
 import sys
@@ -28,6 +31,9 @@ from pylint.checkers import utils
 
 OPEN_FILES = {'open', 'file'}
 UNITTEST_CASE = 'unittest.case'
+THREADING_THREAD = 'threading.Thread'
+COPY_COPY = 'copy.copy'
+OS_ENVIRON = 'os._Environ'
 if sys.version_info >= (3, 0):
     OPEN_MODULE = '_io'
 else:
@@ -105,6 +111,17 @@ class StdlibChecker(BaseChecker):
                   'The method is marked as deprecated and will be removed in '
                   'a future version of Python. Consider looking for an '
                   'alternative in the documentation.'),
+        'W1506': ('threading.Thread needs the target function',
+                  'bad-thread-instantiation',
+                  'The warning is emitted when a threading.Thread class '
+                  'is instantiated without the target function being passed. '
+                  'By default, the first parameter is the group param, not the target param. '),
+        'W1507': ('Using copy.copy(os.environ). Use os.environ.copy() '
+                  'instead. ',
+                  'shallow-copy-environ',
+                  'os.environ is not a dict object but proxy object, so '
+                  'shallow copy has still effects on original object. '
+                  'See https://bugs.python.org/issue15373 for reference. '),
     }
 
     deprecated = {
@@ -117,14 +134,14 @@ class StdlibChecker(BaseChecker):
             'tkinter.Menu.tk_bindForTraversal',
         ],
         2: {
-            (2, 6): [
+            (2, 6, 0): [
                 'commands.getstatus',
                 'os.popen2',
                 'os.popen3',
                 'os.popen4',
                 'macostools.touched',
             ],
-            (2, 7): [
+            (2, 7, 0): [
                 'unittest.case.TestCase.assertEquals',
                 'unittest.case.TestCase.assertNotEquals',
                 'unittest.case.TestCase.assertAlmostEquals',
@@ -137,15 +154,15 @@ class StdlibChecker(BaseChecker):
             ],
         },
         3: {
-            (3, 0): [
+            (3, 0, 0): [
                 'inspect.getargspec',
                 'unittest.case.TestCase._deprecate.deprecated_func',
             ],
-            (3, 1): [
+            (3, 1, 0): [
                 'base64.encodestring', 'base64.decodestring',
                 'ntpath.splitunc',
             ],
-            (3, 2): [
+            (3, 2, 0): [
                 'cgi.escape',
                 'configparser.RawConfigParser.readfp',
                 'xml.etree.ElementTree.Element.getchildren',
@@ -153,42 +170,65 @@ class StdlibChecker(BaseChecker):
                 'xml.etree.ElementTree.XMLParser.getiterator',
                 'xml.etree.ElementTree.XMLParser.doctype',
             ],
-            (3, 3): [
-                'inspect.getmoduleinfo', 'inspect.getmodulename',
+            (3, 3, 0): [
+                'inspect.getmoduleinfo',
                 'logging.warn', 'logging.Logger.warn',
                 'logging.LoggerAdapter.warn',
                 'nntplib._NNTPBase.xpath',
                 'platform.popen',
             ],
-            (3, 4): [
-                'asyncio.tasks.async',
+            (3, 4, 0): [
                 'importlib.find_loader',
                 'plistlib.readPlist', 'plistlib.writePlist',
                 'plistlib.readPlistFromBytes',
                 'plistlib.writePlistToBytes',
-                'xml.etree.ElementTree.iterparse',
             ],
-            (3, 5): [
+            (3, 4, 4): [
+                'asyncio.tasks.async',
+            ],
+            (3, 5, 0): [
                 'fractions.gcd',
-                'inspect.getfullargspec', 'inspect.getargvalues',
+                'inspect.getargvalues',
                 'inspect.formatargspec', 'inspect.formatargvalues',
                 'inspect.getcallargs',
                 'platform.linux_distribution', 'platform.dist',
             ],
+            (3, 6, 0): [
+                'importlib._bootstrap_external.FileLoader.load_module',
+            ],
         },
     }
 
+    def _check_bad_thread_instantiation(self, node):
+        if not node.kwargs and node.args:
+            self.add_message('bad-thread-instantiation', node=node)
+
+    def _check_shallow_copy_environ(self, node):
+        arg = utils.get_argument_from_call(node, position=0)
+        for inferred in arg.inferred():
+            if inferred.qname() == OS_ENVIRON:
+                self.add_message('shallow-copy-environ', node=node)
+                break
+
     @utils.check_messages('bad-open-mode', 'redundant-unittest-assert',
-                          'deprecated-method')
+                          'deprecated-method',
+                          'bad-thread-instantiation',
+                          'shallow-copy-environ')
     def visit_call(self, node):
-        """Visit a CallFunc node."""
+        """Visit a Call node."""
         try:
             for inferred in node.func.infer():
+                if inferred is astroid.Uninferable:
+                    continue
                 if inferred.root().name == OPEN_MODULE:
                     if getattr(node.func, 'name', None) in OPEN_FILES:
                         self._check_open_mode(node)
                 if inferred.root().name == UNITTEST_CASE:
                     self._check_redundant_assert(node, inferred)
+                if isinstance(inferred, astroid.ClassDef) and inferred.qname() == THREADING_THREAD:
+                    self._check_bad_thread_instantiation(node)
+                if isinstance(inferred, astroid.FunctionDef) and inferred.qname() == COPY_COPY:
+                    self._check_shallow_copy_environ(node)
                 self._check_deprecated_method(node, inferred)
         except astroid.InferenceError:
             return
