@@ -1,29 +1,26 @@
-# Copyright (c) 2003-2015 LOGILAB S.A. (Paris, FRANCE).
-# http://www.logilab.fr/ -- mailto:contact@logilab.fr
-#
-# This program is free software; you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the Free Software
-# Foundation; either version 2 of the License, or (at your option) any later
-# version.
-#
-# This program is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# Copyright (c) 2015-2016 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2015 Rene Zhang <rz99@cornell.edu>
+# Copyright (c) 2015 Steven Myint <hg@stevenmyint.com>
+# Copyright (c) 2015 Pavel Roskin <proski@gnu.org>
+# Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
+# Copyright (c) 2016 Derek Gustafson <degustaf@gmail.com>
+
+# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+# For details: https://github.com/PyCQA/pylint/blob/master/COPYING
+
 """Tests for pylint.checkers.exceptions."""
 
 import sys
-import unittest
 
-from astroid import test_utils
+import pytest
+
+import astroid
+
 from pylint.checkers import exceptions
 from pylint.testutils import CheckerTestCase, Message
 
 
-class ExceptionsCheckerTest(CheckerTestCase):
+class TestExceptionsChecker(CheckerTestCase):
     """Tests for pylint.checkers.exceptions."""
 
     CHECKER_CLASS = exceptions.ExceptionsChecker
@@ -33,18 +30,18 @@ class ExceptionsCheckerTest(CheckerTestCase):
     # and `raise (Error, ...)` will be converted to
     # `raise Error(...)`, so it beats the purpose of the test.
 
-    @unittest.skipUnless(sys.version_info[0] == 3,
-                         "The test should emit an error on Python 3.")
+    @pytest.mark.skipif(sys.version_info[0] != 3,
+                        reason="The test should emit an error on Python 3.")
     def test_raising_bad_type_python3(self):
-        node = test_utils.extract_node('raise (ZeroDivisionError, None)  #@')
+        node = astroid.extract_node('raise (ZeroDivisionError, None)  #@')
         message = Message('raising-bad-type', node=node, args='tuple')
         with self.assertAddsMessages(message):
             self.checker.visit_raise(node)
 
-    @unittest.skipUnless(sys.version_info[0] == 2,
-                         "The test is valid only on Python 2.")
+    @pytest.mark.skipif(sys.version_info[0] != 2,
+                        reason="The test is valid only on Python 2.")
     def test_raising_bad_type_python2(self):
-        nodes = test_utils.extract_node('''
+        nodes = astroid.extract_node('''
         raise (ZeroDivisionError, None)  #@
         from something import something
         raise (something, None) #@
@@ -64,6 +61,18 @@ class ExceptionsCheckerTest(CheckerTestCase):
         with self.assertAddsMessages(message):
             self.checker.visit_raise(nodes[3])
 
+    @pytest.mark.skipif(sys.version_info[0] != 3,
+                        reason="The test is valid only on Python 3.")
+    def test_bad_exception_context_function(self):
+        node = astroid.extract_node("""
+        def function():
+            pass
 
-if __name__ == '__main__':
-    unittest.main()
+        try:
+            pass
+        except function as exc:
+            raise Exception from exc  #@
+        """)
+        message = Message('bad-exception-context', node=node)
+        with self.assertAddsMessages(message):
+            self.checker.visit_raise(node)
